"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const core_1 = require("@oclif/core");
const core_2 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const cli_ux_1 = require("cli-ux");
const ts_types_1 = require("@salesforce/ts-types");
const orgs_1 = require("../../utils/orgs");
const componentSetBuilder_1 = require("../../utils/componentSetBuilder");
const output_1 = require("../../utils/output");
const requiredFlagValidator_1 = require("../../utils/requiredFlagValidator");
core_2.Messages.importMessagesDirectory(__dirname);
const messages = core_2.Messages.loadMessages('@salesforce/plugin-deploy-retrieve-metadata', 'retrieve.metadata');
const mdTrasferMessages = core_2.Messages.loadMessages('@salesforce/plugin-deploy-retrieve-metadata', 'metadata.transfer');
// One of these flags must be specified for a valid deploy.
const requiredFlags = ['manifest', 'metadata', 'package-name', 'source-dir'];
class RetrieveMetadata extends sf_plugins_core_1.SfCommand {
    async run() {
        const flags = (await this.parse(RetrieveMetadata)).flags;
        requiredFlagValidator_1.validateOneOfCommandFlags(requiredFlags, flags);
        const componentSet = await componentSetBuilder_1.ComponentSetBuilder.build({
            apiversion: flags['api-version'],
            sourcepath: flags['source-dir'],
            packagenames: flags['package-name'],
            manifest: (flags.manifest && {
                manifestPath: flags.manifest,
                directoryPaths: await orgs_1.getPackageDirs(),
            }),
            metadata: flags.metadata && {
                metadataEntries: flags.metadata,
                directoryPaths: await orgs_1.getPackageDirs(),
            },
        });
        const project = await core_2.SfdxProject.resolve();
        const retrieve = await componentSet.retrieve({
            usernameOrConnection: await orgs_1.resolveTargetOrg(flags['target-org']),
            merge: true,
            output: project.getDefaultPackage().fullPath,
            packageOptions: flags['package-name'],
        });
        if (!flags.json) {
            retrieve.onUpdate((data) => {
                cli_ux_1.cli.action.status = mdTrasferMessages.getMessage(data.status);
            });
            // any thing else should stop the progress bar
            retrieve.onFinish((data) => cli_ux_1.cli.action.stop(mdTrasferMessages.getMessage(data.response.status)));
            retrieve.onCancel((data) => cli_ux_1.cli.action.stop(mdTrasferMessages.getMessage(data.status)));
            retrieve.onError((error) => {
                cli_ux_1.cli.action.stop(error.name);
                throw error;
            });
            cli_ux_1.cli.action.start(messages.getMessage('RetrieveTitle'));
        }
        await retrieve.start();
        const result = await retrieve.pollStatus(500, kit_1.Duration.minutes(flags.wait).seconds);
        const fileResponses = (result === null || result === void 0 ? void 0 : result.getFileResponses()) || [];
        await this.displayResults(result, flags);
        return fileResponses;
    }
    async displayResults(result, flags) {
        if (!ts_types_1.getBoolean(flags, 'json', false)) {
            if (result.response.status === 'Succeeded') {
                output_1.displaySuccesses(result);
                output_1.displayPackages(result, await this.getPackages(result, flags));
            }
            else {
                throw new core_2.SfdxError(ts_types_1.getString(result.response, 'errorMessage', result.response.status), ts_types_1.getString(result.response, 'errorStatusCode', 'unknown'));
            }
        }
    }
    async getPackages(result, flags) {
        const packages = [];
        const projectPath = await core_2.SfdxProject.resolveProjectPath();
        const packageNames = ts_types_1.getArray(flags, 'package-name', []);
        packageNames.forEach((name) => {
            packages.push({ name, path: path.join(projectPath, name) });
        });
        return packages;
    }
}
exports.default = RetrieveMetadata;
RetrieveMetadata.summary = messages.getMessage('summary');
RetrieveMetadata.description = messages.getMessage('description');
RetrieveMetadata.examples = messages.getMessages('examples');
RetrieveMetadata.flags = {
    'api-version': core_1.Flags.string({
        char: 'a',
        summary: messages.getMessage('flags.api-version.summary'),
        description: messages.getMessage('flags.api-version.description'),
    }),
    manifest: core_1.Flags.string({
        char: 'x',
        summary: messages.getMessage('flags.manifest.summary'),
        description: messages.getMessage('flags.manifest.description'),
        exclusive: ['metadata', 'source-dir'],
    }),
    metadata: core_1.Flags.string({
        char: 'm',
        summary: messages.getMessage('flags.metadata.summary'),
        multiple: true,
        exclusive: ['manifest', 'source-dir'],
    }),
    'package-name': core_1.Flags.string({
        char: 'n',
        summary: messages.getMessage('flags.package-name.summary'),
        multiple: true,
    }),
    'source-dir': core_1.Flags.string({
        char: 'd',
        summary: messages.getMessage('flags.source-dir.summary'),
        description: messages.getMessage('flags.source-dir.description'),
        multiple: true,
        exclusive: ['manifest', 'metadata'],
    }),
    'target-org': core_1.Flags.string({
        char: 'o',
        summary: messages.getMessage('flags.target-org.summary'),
        description: messages.getMessage('flags.target-org.description'),
    }),
    wait: core_1.Flags.integer({
        char: 'w',
        default: 33,
        summary: messages.getMessage('flags.wait.summary'),
        description: messages.getMessage('flags.wait.description'),
    }),
};
RetrieveMetadata.configurationVariablesSection = sf_plugins_core_1.toHelpSection('CONFIGURATION VARIABLES', core_2.OrgConfigProperties.TARGET_ORG, core_2.SfdxPropertyKeys.API_VERSION);
RetrieveMetadata.envVariablesSection = sf_plugins_core_1.toHelpSection('ENVIRONMENT VARIABLES', core_2.EnvironmentVariable.SF_TARGET_ORG, core_2.EnvironmentVariable.SFDX_DEFAULTUSERNAME, core_2.EnvironmentVariable.SFDX_USE_PROGRESS_BAR);
//# sourceMappingURL=metadata.js.map